﻿using System.Collections.Immutable;
using Connected.Annotations;
using Connected.Notifications;
using Connected.ServiceModel;

namespace Connected.Common.Types.Continent.Countries;

[Service]
[ServiceUrl(Routes.ContinentCountries)]
public interface IContinentCountryService : IServiceNotifications<int>
{
	[ServiceMethod(ServiceMethodVerbs.Get)]
	Task<ImmutableList<IContinentCountry>?> Query();
	/// <summary>
	/// Queries countries for the specified continent id.
	/// </summary>
	/// <param name="args">The id fo the continent.</param>
	/// <returns>The list of countries.</returns>
	[ServiceMethod(ServiceMethodVerbs.Get | ServiceMethodVerbs.Post)]
	Task<ImmutableList<IContinentCountry>?> QueryCountries(PrimaryKeyArgs<int> args);
	/// <summary>
	/// Select a <see cref="IContinentCountry"/> entity for the specified country id.
	/// </summary>
	/// <param name="args">The country id.</param>
	/// <returns>The <see cref="IContinentCountry"/> entity if exists, <code>null</code> otherwise.</returns>
	[ServiceMethod(ServiceMethodVerbs.Get | ServiceMethodVerbs.Post)]
	Task<IContinentCountry> SelectCountry(PrimaryKeyArgs<int> args);

	[ServiceMethod(ServiceMethodVerbs.Post | ServiceMethodVerbs.Put)]
	Task<int> Insert(InsertContinentCountryArgs args);

	[ServiceMethod(ServiceMethodVerbs.Post | ServiceMethodVerbs.Delete)]
	Task Delete(PrimaryKeyArgs<int> args);
}
