﻿using System.Collections.Immutable;
using Connected.Components;
using Connected.Notifications;
using Connected.ServiceModel;
using Microsoft.AspNetCore.Components;

namespace Connected.Common.Types.TaxRates.Components;

public partial class TaxRatesList : UIComposition
{
	private ImmutableList<ITaxRate>? DataSource { get; set; }

	[Inject]
	private ITaxRateService? TaxRateService { get; set; }
	[Inject]
	private NavigationManager? Navigation { get; set; }

	private string LastInsert { get; set; } = "waiting...";

	protected override async Task OnParametersSetAsync()
	{
		await base.OnParametersSetAsync();

		TaxRateService.Inserted += OnInserted;

		DataSource = await TaxRateService.Query(QueryArgs.Default);
	}

	private void OnInserted(object? sender, PrimaryKeyEventArgs<int> e)
	{
		LastInsert = $"{e.Id} -> {DateTime.UtcNow:F}";
		StateHasChanged();
	}

	private void OnEdit(int id)
	{
		Navigation.NavigateTo(CommonTypesUIRoutes.TaxRatesEdit.Replace("{id:int}", id.ToString()));
	}

	private void OnAdd()
	{
		Navigation.NavigateTo(CommonTypesUIRoutes.TaxRatesAdd);
	}
}
