﻿using Connected.Middleware;
using Connected.Validation;

namespace Connected.Common.Types.Countries;

internal class CountryValidator : MiddlewareComponent, IValidator<InsertCountryArgs>
{
	public CountryValidator(ICountryService countryService)
	{
		CountryService = countryService;
	}

	private ICountryService CountryService { get; }
	/// <summary>
	/// 
	/// </summary>
	/// <param name="args"></param>
	/// <returns></returns>
	public async Task Validate(InsertCountryArgs args)
	{
		/*
		 * Country has the following constraints:
		 * - name is unique
		 * - lcid is unique
		 * - iso2Alpha code is unique
		 */

		/*
		 * This method handles insert and update country methods.
		 * The algorithm is different for each method.
		 * First check if it's an update
		 */
		var update = args as UpdateCountryArgs;

		foreach (var country in await CountryService.Query())
		{
			/*
			 * Ignore self record.
			 */
			if (update is not null && country.Id == update.Id)
				continue;

			if (string.Equals(country.Name, args.Name, StringComparison.OrdinalIgnoreCase))
				throw ValidationExceptions.ValueExists(nameof(country.Name), args.Name);

			if (country.Lcid > 0 && country.Lcid == args.Lcid)
				throw ValidationExceptions.ValueExists(nameof(country.Lcid), args.Lcid);

			if (!string.IsNullOrEmpty(country.IsoCode) && string.Equals(country.IsoCode, args.IsoCode, StringComparison.OrdinalIgnoreCase))
				throw ValidationExceptions.ValueExists(nameof(country.IsoCode), args.IsoCode);
		}
	}
}
