﻿using Connected.Entities;
using Connected.Middleware;
using Connected.Validation;

namespace Connected.Common.Types.TaxRates;

/// <summary>
/// Business logic validator for the <see cref="InsertTaxRateArgs"/> arguments.
/// </summary>
internal sealed class InsertTaxRateValidator : MiddlewareComponent, IValidator<InsertTaxRateArgs>
{
	public InsertTaxRateValidator(ITaxRateCache cache)
	{
		Service = cache;
	}

	private ITaxRateCache Service { get; }

	public async Task Validate(InsertTaxRateArgs args)
	{
		/*
		 * We have a constraint on a name property. 
		 */
		if (await (from dc in Service where string.Equals(dc.Name, args.Name, StringComparison.OrdinalIgnoreCase) select dc).AsEntity() is ITaxRate existing)
			throw ValidationExceptions.ValueExists(nameof(args.Name), args.Name);
	}
}
/// <summary>
/// Business logic validator for the <see cref="UpdateTaxRateArgs"/> arguments.
/// </summary>
internal sealed class UpdateTaxRateValidator : MiddlewareComponent, IValidator<UpdateTaxRateArgs>
{
	public UpdateTaxRateValidator(ITaxRateCache cache)
	{
		Service = cache;
	}

	private ITaxRateCache Service { get; }

	public async Task Validate(UpdateTaxRateArgs args)
	{
		/*
		 * A difference from the insert validator is to exclude the id of the updating record.
		 */
		if (await (from dc in Service where string.Equals(dc.Name, args.Name, StringComparison.OrdinalIgnoreCase) && dc.Id != args.Id select dc).AsEntity() is ITaxRate existing)
			throw ValidationExceptions.ValueExists(nameof(args.Name), args.Name);
	}
}
