﻿using System.Collections.Immutable;
using Connected.Entities;
using Connected.ServiceModel;
using static Connected.Common.Types.TaxRates.TaxRateOps;

namespace Common.Types.TaxRates.Ops;


[TestClass]
public class QueryTests
{
	[TestMethod("Query invoke returns ImmutableList when set is empty")]
	public async Task Query_Invoke_ReturnsImmutableList_WhenEmpty()
	{
		/* 
		 * Setup
		 */
		var dataSet = TaxRateSamples.GetEmptyTaxRateSet();

		var instanceFaker = new InstanceFaker<Query>();

		var taxRateCacheFake = instanceFaker.GetMock<ITaxRateCache>()!;

		taxRateCacheFake.SetupIEnumerable(dataSet!);

		var args = QueryArgs.Default;
		/* 
		 * Test
		 */
		var serviceFunction = instanceFaker.Instance;
		var result = await serviceFunction.Invoke(args);
		/* 
		 * Assert
		 */
		Assert.IsNotNull(result);
		Assert.IsTrue(result.GetType().IsAssignableToGenericType(typeof(ImmutableList<>)));
	}

	[TestMethod("Query invoke returns all entries when no entries are deleted")]
	public async Task Query_Invoke_ReturnsAllEntries_WhenNoEntriesAreDeleted()
	{
		/*
		 * Setup
		 */
		var dataSet = TaxRateSamples.GetTaxRates(State.Default)
			.Union(TaxRateSamples.GetTaxRates(State.New));

		var instanceFaker = new InstanceFaker<Query>();

		var taxRateCacheFake = instanceFaker.GetMock<ITaxRateCache>()!;

		taxRateCacheFake.SetupIEnumerable(dataSet!);

		var args = Dto.Empty;
		/* 
		 * Test
		 */
		var serviceFunction = instanceFaker.Instance;
		var result = await serviceFunction.Invoke(QueryArgs.Default);
		/* 
		 * Assert
		 */
		Assert.IsNotNull(result);
		Assert.IsTrue(result.SequenceEqual(dataSet));
	}

	[TestMethod("Query invoke returns only non deleted entries when entries are deleted")]
	public async Task Query_Invoke_ReturnsNonDeletedOnly_WhenEntriesAreDeleted()
	{
		/* 
		 * Setup
		*/
		var expectedDataSet = TaxRateSamples.GetTaxRates(State.Default)
			.Union(TaxRateSamples.GetTaxRates(State.New));

		var deletedDataSet = expectedDataSet.Union(TaxRateSamples.GetTaxRates(State.Deleted));

		var instanceFaker = new InstanceFaker<Query>();

		var taxRateCacheFake = instanceFaker.GetMock<ITaxRateCache>()!;

		taxRateCacheFake.SetupIEnumerable(deletedDataSet);

		var args = Dto.Empty;
		/* 
		 * Test
		 */
		var serviceFunction = instanceFaker.Instance;
		var result = await serviceFunction.Invoke(QueryArgs.Default);
		/* 
		 * Assert
		 */
		Assert.IsNotNull(result);
		Assert.IsTrue(result.SequenceEqual(expectedDataSet));
	}
}